﻿using System;
using UnityEngine;
using System.Collections;

public class MainMenu : MonoBehaviour
{
    private bool pVisible = false;
    private Animator anim;
    private bool animRunning = false;
    private int stateHashOut;
    private int stateHashIn;
    internal bool inAnimation = false;

    public event EventHandler AnimationDone;

    public TypewriterEffect typeWriterEffect;

    internal bool IsVisible
    {
        get
        {
            return pVisible;
        }
        set
        {
            if (pVisible != value)
            {
                pVisible = value;
                if (anim != null)
                    anim.SetBool("Visible", pVisible);

                if (pVisible)
                {
                    typeWriterEffect.ResetToBeginning();
                }
            }
        }
    }

    void Start()
    {
        anim = GetComponent<Animator>();
        stateHashOut = Animator.StringToHash("MenuOut");
        stateHashIn = Animator.StringToHash("MenuIn");
        anim.Play(stateHashOut, 0, 1);
        anim.CrossFade(stateHashOut, 0, 0, 1);
        inAnimation = false;
    }

    void Update()
    {
        AnimatorStateInfo asi = anim.GetCurrentAnimatorStateInfo(0);        
        //Debug.Log(asi.normalizedTime);
        if (asi.normalizedTime >= 1)
        {
            if (inAnimation)
            {
                inAnimation = false;
                if (this.AnimationDone != null)
                {
                    this.AnimationDone(this, EventArgs.Empty);
                }
            }
        }
        else if (asi.normalizedTime > 0.1f)
        {
            if (!inAnimation)
            {
                inAnimation = true;
            }
        }
        //Debug.Log(inAnimation);
    }

    public void OnPlayClick()
    {
        Game.Current.state = "Play";
        Game.Current.showMainMenu = false;
    }

    public void OnOptionsClick()
    {
        Game.Current.state = "Options";
        Game.Current.showMainMenu = false;
    }

    public void OnExitClick()
    {
        Game.Current.state = "Exit";
        Game.Current.showMainMenu = false;
    }
}
