﻿using System.ComponentModel;
using LibNoise;
using Unity.PerForm.Utils;
using UnityEngine;
using System.Collections;
using System;

[ExecuteInEditMode]
[RequireComponent(typeof(Animator))]
public class AnimationTool : MonoBehaviour
{
    public enum AnimRandomOutMode
    {
        Mix,
        Larger,
        Smaller,
        Raising,
        Falling
    }

	[HideInInspector]
    [NonSerialized]
	public Animator anim;

    private float anSpeed = 0;
    private int anStHash;    
    private bool isInit = false;

    [Range(0.0f, 1.0f)]
    public float animationPosition = 0.0f;

    public bool limitOutputValue = true;

    [HideInInspector]
    public bool enablePositionCurve = false;    

    [HideInInspector]
    public AnimationCurve positionCurve = AnimationCurve.Linear(0.0f, 0.0f, 1.0f, 1.0f);

    [HideInInspector]
    public bool positionCurvesRandom = false;

    [HideInInspector] 
    public bool randomizeOnStart = false;

    [HideInInspector] 
    public AnimRandomOutMode randomOutMode = AnimRandomOutMode.Mix;

    [HideInInspector]
    public AnimationCurve randomCurveMix = AnimationCurve.Linear(0.0f, 0.5f, 1.0f, 0.5f);

    [HideInInspector]
    public AnimationCurve randomPositionCurve = AnimationCurve.Linear(0.0f, 0.0f, 1.0f, 1.0f);

    [HideInInspector] 
    [Range(2, 20)]
    public int randomCurveMixPoints = 2;

    private float currentAnimPos = 0;

    private AnimationCurve animCurve;

    public void CreateMixCurve(bool centered)
    {

        if (centered)
        {            
            randomCurveMix = AnimationCurve.Linear(0.0f, 0.0f, 1.0f, 1.0f);
            return;            
        }


        if (randomCurveMixPoints == 1)
        {
            float rv = UnityEngine.Random.Range(0.0f, 1.0f);
            randomCurveMix = AnimationCurve.Linear(0.0f, rv, 1.0f, rv);

            return;
        }

        int nrk = randomCurveMixPoints;
        if (nrk < 2)
        {
            nrk = 2;
        }


        float cp = 0;
        float np;
		float crv = UnityEngine.Random.Range(0.0f, 1.0f);
        float nrv;

        float kt = 1.0f / (float)(nrk - 1);
        Keyframe[] nkeys = new Keyframe[nrk];
        for (int i = 0; i <= nrk; i++)
        {
            np = cp;            
			nrv = UnityEngine.Random.Range(0.0f, 1.0f);

            if (i < nrk)
            {
                Keyframe kf = new Keyframe(np, crv);
                kf.value = crv;

                nkeys[i] = kf;
            }
            
            randomCurveMix = new AnimationCurve(nkeys);

            cp = (i + 1) * kt;
            crv = nrv;
        }        
    }

    public float ComputeAnimTime(float pos)
    {
        if (enablePositionCurve)
        {
            if (positionCurvesRandom)
            {
                float mv = randomCurveMix.Evaluate(pos);
                float mvi = 1.0f - mv;

                float v1 = positionCurve.Evaluate(pos);
                float v2 = randomPositionCurve.Evaluate(pos);


                switch (randomOutMode)
                {
                    case AnimRandomOutMode.Larger:
                        return Misc.LargerValue(v1, v2);
                    case AnimRandomOutMode.Smaller:
                        return Misc.SmallerValue(v1, v2);
                    default:
                        return v1 * mvi + v2 * mvi;
                }
                

            }
            else
            {
                return Misc.LimitValue(positionCurve.Evaluate(pos), 0.0f, 1.0f);
            }
        }

        return pos;
    }

	void OnEnable()
	{

        if (randomizeOnStart)
        {
            CreateMixCurve(false);
        }

	    anim = GetComponent<Animator>();
        		anim.speed = 0;
        
		isInit = anim.runtimeAnimatorController != null;
		if (isInit) {
		        
				//AnimationInfo[] ais = anim.GetCurrentAnimationClipState(0);
				//if (ais.Length == 0) {
				//	isInit = false;
				//} else {

					anStHash = anim.GetCurrentAnimatorStateInfo(0).nameHash;
					anim.Play(anStHash, 0, 0);

				//}
		}

	    
	}

    void OnDisable()
    {
        isInit = false;

        anim = null;
    }
	
	void Update ()
	{
	    if (!isInit) {				
			if (Application.isEditor && !Application.isPlaying) {
				anim = GetComponent<Animator>();
				anim.speed = 0;
				
				isInit = anim.runtimeAnimatorController != null;
				if (isInit) {
					
					//AnimationInfo[] ais = anim.GetCurrentAnimationClipState(0);
					//if (ais.Length == 0) {
					//	isInit = false;
					//} else {
					
					anStHash = anim.GetCurrentAnimatorStateInfo(0).nameHash;
					//anim.Play(anStHash, 0, 0);
					
					//}
				}
			}

			return;
		} else {
			//animation.clip.frameRate = 0;
			//animation.Play(anClip.name, PlayMode.
			float ctim = ComputeAnimTime(animationPosition);

			anim.Play(anStHash, 0, ctim);

			//gameObject.SampleAnimation(
			//Debug.Log(animation.isPlaying + "; " + anim.GetCurrentAnimatorStateInfo(0).normalizedTime);
		}

	}
}