﻿using System;
using System.ComponentModel;
using UnityEngine;
using System.Collections;
using Unity.PerForm.Utils;
using Random = UnityEngine.Random;

//[ExecuteInEditMode]
[RequireComponent(typeof(Animation))]
public class AnimationTool : MonoBehaviour
{
    [HideInInspector]
    [NonSerialized]
    public bool editorPeviewEnabled = false;


    public enum AnimRandomOutMode
    {
        Mix,
        Larger,
        Smaller,
        Raising,
        Falling
    }

    //private float anSpeed = 0;
    private AnimationClip anClip;
    private AnimationState anState;
    private bool isInit = false;
    private bool mixRandomized = false;

    [Range(0.0f, 1.0f)]
    public float animationPosition = 0.0f;

    public bool limitOutputValue = true;

    [HideInInspector]
    public bool enablePositionCurve = false;    

    [HideInInspector]
    public AnimationCurve positionCurve = AnimationCurve.Linear(0.0f, 0.0f, 1.0f, 1.0f);

    [HideInInspector]
    public bool positionCurvesRandom = false;

    [HideInInspector] 
    public bool randomizeOnStart = false;

    [HideInInspector] 
    public AnimRandomOutMode randomOutMode = AnimRandomOutMode.Mix;

    [HideInInspector]
    public AnimationCurve randomCurveMix = AnimationCurve.Linear(0.0f, 0.5f, 1.0f, 0.5f);

    [HideInInspector]
    public AnimationCurve randomPositionCurve = AnimationCurve.Linear(0.0f, 0.0f, 1.0f, 1.0f);

    [HideInInspector] 
    [Range(2, 20)]
    public int randomCurveMixPoints = 2;

    //private float currentAnimPos = 0;

    private AnimationCurve animCurve;

    public void CreateMixCurve(bool centered)
    {

        if (centered)
        {            
            randomCurveMix = AnimationCurve.Linear(0.0f, 0.0f, 1.0f, 1.0f);
            return;            
        }


        if (randomCurveMixPoints == 1)
        {
            float rv = Random.Range(0.0f, 1.0f);
            randomCurveMix = AnimationCurve.Linear(0.0f, rv, 1.0f, rv);

            return;
        }

        int nrk = randomCurveMixPoints;
        if (nrk < 2)
        {
            nrk = 2;
        }


        float cp = 0;
        float np;
        float crv = Random.Range(0.0f, 1.0f);
        float nrv;

        float kt = 1.0f / (float)(nrk - 1);
        Keyframe[] nkeys = new Keyframe[nrk];
        for (int i = 0; i <= nrk; i++)
        {
            np = cp;            
            nrv = Random.Range(0.0f, 1.0f);

            if (i < nrk)
            {
                Keyframe kf = new Keyframe(np, crv);
                kf.value = crv;

                nkeys[i] = kf;
            }
            
            randomCurveMix = new AnimationCurve(nkeys);

            cp = (i + 1) * kt;
            crv = nrv;
        }        
    }

    /*void OnGUI()
    {
        Handles.Label(transform.position, animation[anState.name].normalizedTime.ToString("n3"));
    }*/

    public float ComputeAnimTime(float pos)
    {
        float clpos = Misc.LimitValue(pos, 0.0f, 0.999f);
        //float anpos = clpos;

        if (enablePositionCurve)
        {
            if (positionCurvesRandom)
            {

                float mv = Misc.LimitValue(randomCurveMix.Evaluate(clpos), 0.0f, 1.0f);
                float mvi = 1.0f - mv;

                float v1 = positionCurve.Evaluate(clpos);
                float v2 = randomPositionCurve.Evaluate(clpos);


                switch (randomOutMode)
                {
                    case AnimRandomOutMode.Larger:
                        return Misc.LargerValue(v1, v2);
                    case AnimRandomOutMode.Smaller:
                        return Misc.SmallerValue(v1, v2);
                    default:
                        return v1 * mvi + v2 * mv;
                }
                

            }
            else
            {
                //return Misc.LimitValue(positionCurve.Evaluate(clpos), 0.0f, 1.0f);
            }
        }

        return clpos;
    }

    void Start()
    {

        if (randomizeOnStart && !mixRandomized)
        {
            mixRandomized = true;
            CreateMixCurve(false);
        }

        anClip = GetComponent<Animation>().clip;
        anState = GetComponent<Animation>()[anClip.name];
        //anSpeed = anState.speed;
        anState.speed = 0;



        if (!GetComponent<Animation>().isPlaying)
            GetComponent<Animation>().Play(anState.clip.name);
        
        //anState.speed = 0.0f;
        anState.normalizedTime = 0;

        isInit = anClip != null;
    }


    void OnDisable()
    {
        isInit = false;

        //anState.speed = anSpeed;
        //anState = null;
        //anClip = null;
    }

    public void SetAnimationPosition(float apos)
    {
        if (!isInit)
        {
            anState = GetComponent<Animation>()[anClip.name];
            anState.speed = 0;
            anState.normalizedTime = ComputeAnimTime(apos);
            //Debug.Log(animation.isPlaying + "; " + anState.normalizedTime);
        }
        else
        {
            anState.normalizedTime = Misc.LimitValue(ComputeAnimTime(Misc.Clamp(apos)), 0.0f, 0.999f);
        }
    }

	void Update ()
	{
        SetAnimationPosition(animationPosition);
	}
}