﻿using UnityEngine;
using System;
using System.Collections;
using Unity.PerForm.Utils;

[ExecuteInEditMode]
public class LookAtObject : MonoBehaviour {

	public bool enableInEditor = false;

	public Transform targetObject;
	private bool hasTargetObject = false;

	public Vector3 positionAdjust = new Vector3();

	public bool doLerp = false;

	[Range(0.0f, 30.0f)]
	public float lerpTime = 1.0f;

	[HideInInspector]
	[NonSerialized]
	public Quaternion cRot = Quaternion.identity;

	public bool animationByDistance = false;

	private int animationByDistanceState = -1;

	public Animator distanceAnimationController;


	[Range(0.0f, 200.0f)]
	public float animationDistanceRangeFrom = 0.0f;

	[Range(0.0f, 200.0f)]
	public float animationDistanceRangeTo = 0.0f;

	[HideInInspector]
	[NonSerialized]
	public float animationDistanceRange = 0.0f;

	[HideInInspector]
	[NonSerialized]
	public float cDist = 0;

	[HideInInspector]
	[NonSerialized]
	public Vector3 dir = new Vector3();

	void OnEnable() {
		hasTargetObject = targetObject != null;

		if (!hasTargetObject)
			return;

		dir = targetObject.transform.position - (transform.position + positionAdjust);
		cRot = Quaternion.LookRotation(dir);

		if (animationByDistance && distanceAnimationController == null)
			distanceAnimationController = GetComponent<Animator>();

		animationByDistance &= distanceAnimationController != null;
		if (animationByDistance) {
			animationByDistanceState = distanceAnimationController.GetCurrentAnimatorStateInfo(0).nameHash;
			distanceAnimationController.speed = 0;
			distanceAnimationController.StartPlayback();
			//distanceAnimationController.Play(animationByDistanceState, 0, 0);
		}

		animationDistanceRange = Mathf.Abs(animationDistanceRangeTo - animationDistanceRangeFrom);
	}

	void OnDisable() {
		if (distanceAnimationController) {

		}
	}

	public void UpdateRot() {
		if (!hasTargetObject) {
			if (Application.isEditor & !Application.isPlaying) {
				hasTargetObject = targetObject != null;
			}
			return;
		}

		if (Application.isEditor && !Application.isLoadingLevel && !enableInEditor)
			return;

		dir = targetObject.transform.position - (transform.position + positionAdjust);
		if (doLerp && Application.isPlaying) {
			cRot = Quaternion.Lerp(cRot, Quaternion.LookRotation(dir), Time.deltaTime * lerpTime);
		} else {
			cRot = Quaternion.LookRotation(dir);
		}

		transform.rotation = cRot;

		if (animationByDistance) {
			#if UNITY_EDITOR
			if (!Application.isPlaying) {
				if (animationByDistance && distanceAnimationController == null)
					distanceAnimationController = GetComponent<Animator>();

				animationByDistance &= distanceAnimationController != null;
				if (animationByDistance) {
					animationByDistanceState = distanceAnimationController.GetCurrentAnimatorStateInfo(0).nameHash;
					distanceAnimationController.speed = 0;
					distanceAnimationController.Play(animationByDistanceState, 0, 0);
				}
			} 
			#endif
				

			cDist = Vector3.Distance(gameObject.transform.position, targetObject.transform.position);
			
			cDist = Misc.LimitValue(cDist, animationDistanceRangeFrom, animationDistanceRangeTo) / animationDistanceRangeTo;
			distanceAnimationController.Play(animationByDistanceState, 0, cDist);
			//distanceAnimationController.playbackTime = cDist;

		}
	}

	void Update () {
		UpdateRot();
	}
}
