﻿using System;
using System.Collections;
using System.Globalization;

namespace Unity.PerForm.Utils
{
    public sealed class Conv
    {

        public const float degToRad = (float)(Math.PI / 180f);
        public const float radToDeg = (float)(180f / Math.PI);

        public const float PI = (float)(Math.PI);
        public const float PI12 = (float)(Math.PI) * 0.5f;
        public const float PI14 = (float)(Math.PI) * 0.25f;
        public const float PI34 = (float)(Math.PI) * 1.5f;
        public const float PI2 = (float)(Math.PI) * 2f;

        public static byte[] SingleData(float v)
        {            
            return BitConverter.GetBytes(v);
        }

        public static float SingleFromData(byte[] vData)
        {
            return BitConverter.ToSingle(vData, 0);
        }


        public static float RadToDeg(float rad)
        {
            return rad * radToDeg;
        }

        public static float DegToRad(float deg)
        {
            return deg * degToRad;
        }

        public static int StringToInt(string val)
        {
            double tv = 0;
            int rv = 0;

            if (double.TryParse(val, NumberStyles.Integer, CultureInfo.CurrentCulture.NumberFormat, out tv))
            {
                rv = (int)tv;
            }

            return rv;
        }

        public static string IntToString(int val, int digits)
        {
            string sv = Math.Abs(val).ToString();
            if (sv.Length < digits)
            {
                sv = new string('0', digits - sv.Length) + sv;
            }
            if (val < 0) sv = "-" + sv;
            return sv;
        }

        public static object StringToEnum(Type enumType, string eval)
        {
            string[] enNamesTemp = Enum.GetNames(enumType);
            SortedList enItems = new SortedList();
            Array enValsTemp = Enum.GetValues(enumType);
            for (int i = 0; i < enNamesTemp.Length; i++)
            {
                enItems.Add(enNamesTemp[i].ToLower(), enValsTemp.GetValue(i));
            }
            enNamesTemp = null;

            object rv = enItems[eval.ToString()];

            if (rv == null)
            {
                rv = enValsTemp.GetValue(0);
            }

            enValsTemp = null;

            return rv;
        }


        public static float StringToSigle(string val)
        {
            //float rv = 0;

            string cs = CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
            char dc = cs[0];

            string sv = Misc.GetCurCultureNumberString(val);
            int dpi = sv.IndexOf(dc);
            int dpni = sv.IndexOf(dc, dpi + 1);
            if (dpni > 0)
            {
                sv = sv.Substring(0, dpni);
            }

            double rv = 0;

            if (double.TryParse(sv, NumberStyles.Float, CultureInfo.CurrentCulture.NumberFormat, out rv))
            {
                return (float)rv;
            }
            else
            {
                return 0;
            }

            return (float)rv;
        }

        public static string StringArrayToCommSep(string[] vals)
        {
            string rv = "";
            foreach (string v in vals)
            {
                rv = v + ",";
            }
            return rv;
        }

        public static string NullStringToString(object val)
        {
            return NullStringToString(val, "NULL");
        }

        public static string NullStringToString(object val, string nullText)
        {
            if (val == null)
            {
                return nullText;
            }
            else if (val == DBNull.Value)
            {
                return nullText;
            }
            return val.ToString();
        }

        public static byte[] StringEToByteArray(string val)
        {
            byte[] sd = new byte[val.Length + 1];
            char[] scd = val.ToCharArray();
            for (int i = 0; i < scd.Length; i++)
            {
                sd[i] = (byte)scd[i];
            }
            sd[sd.Length - 1] = (byte)'\0';
            scd = null;
            return sd;
        }

        public static string DateTimeDBNullToPXPDateString(object val, bool outputTime, bool shortMode)
        {
            if (Null.IsNull(val))
            {
                return "";
            }
            DateTime dt = Convert.ToDateTime(val);
            string ds;
            if (shortMode)
            {
                ds = dt.ToString("yyyyMMdd");
            }
            else
            {
                ds = dt.ToString("yyyy-MM-dd");
            }
            if (outputTime)
            {
                ds += " ";
                if (shortMode)
                {
                    ds += dt.ToString("HHmmss");
                }
                else
                {
                    ds += dt.ToString("HH:mm:ss");
                }
            }
            return ds;
        }


        public static string DateTimeDBNullToPXPDateString(object val)
        {
            return DateTimeDBNullToPXPDateString(val, false, false);
        }

        public static string IntDBNullToString(object val)
        {
            if (Null.IsNull(val))
            {
                return "";
            }
            return Null.ToInt(val).ToString();
        }

        public static string[] ArrayListToStringArray(ArrayList sData)
        {
            string[] sArr = new string[sData.Count];
            int si = 0;
            foreach (string s in sData)
            {
                sArr[si] = s;
                si++;
            }
            return sArr;
        }

        public static int TryStringToInt(string sval)
        {
            return TryStringToInt(sval, 0);
        }

        public static int TryStringToInt(string sval, int defVal)
        {
            double res = 0;
            if (double.TryParse(sval, NumberStyles.Integer, CultureInfo.CurrentCulture.DateTimeFormat, out res))
            {
                return Convert.ToInt32(res);
            }

            return defVal;
        }

        public static string StringToStringNull(string val)
        {
            if (val == null)
                return null;

            if (val.Length == 0)
            {
                return null;
            }
            else
            {
                return val;
            }
        }

        public static object StringToStringDBNull(string val)
        {
            if (val.Length == 0)
            {
                return DBNull.Value;
            }
            else
            {
                return val;
            }
        }

        public static string SingleDBNullToString(object val)
        {
            return SingleDBNullToString(val, -1);
        }

        public static string SingleDBNullToString(object val, int roundDec)
        {
            if (Null.IsNull(val))
            {
                return "";
            }
            float cv = Convert.ToSingle(val);
            if (roundDec == -1)
            {
                return cv.ToString();
            }
            else
            {
                return Math.Round(cv, roundDec).ToString();
            }
        }

        public static object StringToIntNull(string strVal)
        {
            strVal = strVal.Trim();
            if (strVal.Length == 0)
            {
                return DBNull.Value;
            }
            return Convert.ToInt32(strVal);
        }

        public static object StringToFloatNull(string strVal)
        {
            strVal = strVal.Trim();
            if (strVal.Length == 0)
            {
                return DBNull.Value;
            }

            char ds = CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator[0];
            strVal = strVal.Replace(',', ds).Replace('.', ds);

            return Convert.ToSingle(strVal);
        }

        public static object StringToDecimalNull(string strVal)
        {
            strVal = strVal.Trim();
            if (strVal.Length == 0)
            {
                return DBNull.Value;
            }

            char ds = CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator[0];
            strVal = strVal.Replace(',', ds).Replace('.', ds);

            return Convert.ToDecimal(strVal);
        }

        public static object PXPDateStrToDateTimeNull(string dateStr)
        {
            if (dateStr == null)
                return null;

            string[] dParts = dateStr.Split(new char[] { '-' });
            if (dParts.Length == 3)
            {
                return PXPDateStrToDateTime(dateStr);
            }

            return null;
        }

        public static DateTime? PXPDateStrToDateTime(string dateStr)
        {
            int th = 0;
            int tm = 0;
            int ts = 0;

            if (dateStr.Contains(":"))
            {
                int ti = dateStr.IndexOf(' ');
                string timeStr = dateStr.Substring(ti + 1);
                string[] tStrParts = timeStr.Split(new char[] { ':', '.' });
                th = Conv.TryStringToInt(tStrParts[0], 0);
                tm = Conv.TryStringToInt(tStrParts[1], 0);
                if (tStrParts.Length > 2)
                {
                    ts = Conv.TryStringToInt(tStrParts[2], 0);
                }
                dateStr = dateStr.Substring(0, ti);
            }
            string[] dParts = dateStr.Split(new char[] { '-' });
            if (dParts.Length == 3)
            {
                int y = Convert.ToInt32(dParts[0]);
                int m = Convert.ToInt32(dParts[1]);
                int d = Convert.ToInt32(dParts[2]);
                DateTime rd = new DateTime(y, m, d, th, tm, ts);
                return rd;
            }
            else
            {
                return null;
            }
        }

        public static bool BoolFromString(string val)
        {
            return BoolFromString(val, false);
        }

        public static bool BoolFromString(string val, bool defaultValue)
        {
            string vs = val.ToLower().Trim();
            if (vs == "true")
            {
                return true;
            }
            else if (vs == "false")
            {
                return false;
            }
            return defaultValue;
        }

        public static bool BoolFromInt(int val)
        {
            if (val == 1)
            {
                return true;
            }
            return false;
        }

        public static int IntFromBool(bool val)
        {
            if (val)
            {
                return 1;
            }
            else
            {
                return 0;
            }
        }

        public static string DBNullString(object val)
        {
            string res = "";
            if (val != null)
            {
                if (val != DBNull.Value)
                {
                    res = val.ToString();
                }
            }
            return res;
        }

        public static char[] CharArrayFromByteArray(byte[] data)
        {
            char[] ret = new char[data.Length];
            for (int i = 0; i < data.Length; i++)
            {
                ret[i] = (char)data[i];
            }
            return ret;
        }

        public static string StringFromByteArray(byte[] data)
        {
            char[] cdata = CharArrayFromByteArray(data);
            string retStr = new string(cdata);
            cdata = null;
            return retStr;
        }

        public static byte[] ByteArrayFromCharArray(char[] data)
        {
            byte[] ret = new byte[data.Length];
            for (int i = 0; i < data.Length; i++)
            {
                ret[i] = (byte)data[i];
            }
            return ret;
        }

        public static byte[] ByteArrayFromString(string val)
        {
            char[] valcd = val.ToCharArray();
            byte[] vald = ByteArrayFromCharArray(valcd);
            valcd = null;
            return vald;
        }

        public static byte GetHexByte(char val)
        {
            val = Char.ToUpper(val);
            switch (val)
            {
                case '0':
                    return 0;
                case '1':
                    return 1;
                case '2':
                    return 2;
                case '3':
                    return 3;
                case '4':
                    return 4;
                case '5':
                    return 5;
                case '6':
                    return 6;
                case '7':
                    return 7;
                case '8':
                    return 8;
                case '9':
                    return 9;
                case 'A':
                    return 10;
                case 'B':
                    return 11;
                case 'C':
                    return 12;
                case 'D':
                    return 13;
                case 'E':
                    return 14;
                case 'F':
                    return 15;
                default:
                    return 0;
            }
        }

        public static char GetHexChar(byte val)
        {
            switch (val)
            {
                case 0:
                    return '0';
                case 1:
                    return '1';
                case 2:
                    return '2';
                case 3:
                    return '3';
                case 4:
                    return '4';
                case 5:
                    return '5';
                case 6:
                    return '6';
                case 7:
                    return '7';
                case 8:
                    return '8';
                case 9:
                    return '9';
                case 10:
                    return 'A';
                case 11:
                    return 'B';
                case 12:
                    return 'C';
                case 13:
                    return 'D';
                case 14:
                    return 'E';
                case 15:
                    return 'F';
                default:
                    return ' ';
            }
        }

        public static byte[] HexStringToByteArray(string val)
        {
            byte[] hsdata = new byte[val.Length / 2];
            for (int i = 0; i < hsdata.Length; i++)
            {
                byte cbv = HexStringToByte(val.Substring(i * 2, 2));
                hsdata[i] = cbv;
            }
            return hsdata;
        }

        public static byte HexStringToByte(string val)
        {
            if (val.Length == 0)
                return 0;
            if (val.Length == 1)
                val = "0" + val;
            byte lb = GetHexByte(val[1]);
            byte hb = Convert.ToByte(GetHexByte(val[0]) << 4);
            return Convert.ToByte(hb | lb);
        }

        public static string ByteToHexString(byte val)
        {
            byte bl = Convert.ToByte(val & 15);
            byte bh = Convert.ToByte((val & 240) >> 4);
            return new string(new char[] { GetHexChar(bh), GetHexChar(bl) });
        }

        public static string ByteArrayToHexString(byte[] data)
        {
            string hd = "";
            for (int i = 0; i < data.Length; i++)
            {
                hd += ByteToHexString(data[i]);
            }
            return hd;
        }

    }

}
