﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.IO.Compression;
using System.Text;
using UnityEngine;

namespace Unity.PerForm.Utils
{
    public sealed class Misc
    {
        public static Vector3 InterpolateFromVectorArray(float pos, ref Vector3[] values)
        {
            int fvIndex = (int)Mathf.Floor((values.Length - 1) * pos);
            int nvIndex = fvIndex + 1;
            if (nvIndex >= values.Length)
            {
                nvIndex = values.Length - 1;
                fvIndex = nvIndex - 1;
            }

            float pSize = 1.0f / (float)(values.Length - 1);
            float cvPos = Misc.MapValue(pos, fvIndex * pSize, nvIndex * pSize, 0, 1);

            Vector3 v1 = values[fvIndex];
            Vector3 v2 = values[nvIndex];			            
			return new Vector3(
				v1.x * (1 - cvPos) + cvPos * v2.x,
				v1.y * (1 - cvPos) + cvPos * v2.y,
				v1.z * (1 - cvPos) + cvPos * v2.z
			);
        }

        public static float InterpolateFromArray(float pos, ref float[] values)
        {
            int fvIndex = (int)Mathf.Floor((values.Length - 1) * pos);
            int nvIndex = fvIndex + 1;
            if (nvIndex >= values.Length)
            {
                nvIndex = values.Length - 1;
                fvIndex = nvIndex - 1;
            }

            float pSize = 1.0f / (float)(values.Length - 1);
            float cvPos = Misc.MapValue(pos, fvIndex * pSize, nvIndex * pSize, 0, 1);

			float v1 = values [fvIndex];
			float v2 = values [nvIndex];

			return v1*(1 - cvPos) + cvPos * v2;

			//UnityEngine.Debug.Log ("CP: " + cvPos.ToString("n2") + ", V1 : " + (values [fvIndex]).ToString ("n2") + ", V2: " + (values [nvIndex]).ToString ("n2") + ", R: " + vres.ToString ("n2"));
			//return vres;
        }

		public static float Clamp(float value) {
			if (value < 0)
				return 0;
			if (value > 1.0f)
				return 1.0f;
			return value;
		}

		public static Vector2 Vector2From3(Vector3 v) {
			return new Vector2 (v.x, v.y);
		}

		public static float SmallerValue(float v1, float v2) {
			if (v1 < v2)
				return v1;

			return v2;
		}


		public static float LargerValue(float v1, float v2) {
			if (v1 > v2)
				return v1;

			return v2;
		}

		public static Vector3 Vector3Add(Vector3 a, Vector3 b) {
			return new Vector3 (a.x + b.x, a.y + b.y, a.z + b.z);
		}

		public static Vector3 Vector3AddX(Vector3 a, float v) {
			return new Vector3 (a.x + v, a.y, a.z);
		}

		public static Vector3 Vector3AddY(Vector3 a, float v) {
			return new Vector3 (a.x, a.y + v, a.z);
		}

		public static Vector3 Vector3AddZ(Vector3 a, float v) {
			return new Vector3 (a.x, a.y, a.z + v);
		}

        public static float PhaseValue(float val, float len)
        {
            if (val == 0)
                return 0;

            float nv = val % len;
            if (nv <= 0)
                nv = len + nv;
            return nv;
        }


        public static int PhaseValue(int val, int len)
        {
            int nv = val % len;
            if (nv < 0)
                nv = len + nv;
            return nv;
        }

        public static float TensionValue(float val, float wanted, float speed)
        {
            return TensionValue(val, wanted, speed, 0.01f);
        }
        
        public static float TensionValue(float val, float wanted, float speed, float minValue)
        {
            float nv = val;
            float vd = val - wanted;

            if (vd < -minValue)
            {
                nv -= vd * speed;
            } else if (vd > minValue)
            {
                nv -= vd * speed;
            }
            return nv;
        }        

        public static int FindDataIndex(ref List<byte> data, ref byte[] fData, int startIndex)
        {
            bool isEqu = false;
            for (int si = startIndex; si < data.Count - (fData.Length - 1); si++)
            {
                isEqu = true;
                for (int i = 0; i < fData.Length; i++)
                {
                    if (data[si + i] != fData[i])
                    {
                        isEqu = false;
                        break;
                    }
                }
                if (isEqu)
                    return si;
            }

            return -1;
        }

        public static int CountLines(ref string testStr)
        {
            int lc = 0;

            int ci = testStr.IndexOf('\r');
            while (ci != -1)
            {
                lc++;
                ci = testStr.IndexOf('\r', ci + 1);
            }            

            return lc;
        }

        public static string FindFreeNumberedFile(string filePattern, string filePath)
        {
            string fn = Misc.GetFileName(filePath);
            double cfn = 0;
            if (double.TryParse(fn, NumberStyles.Integer, CultureInfo.InvariantCulture.NumberFormat, out cfn))
            {
                int cfi = Convert.ToInt32(cfn);

            }
            return filePath;
        }

        public static string PutLongStringToLines(string lStr, int lineLength)
        {
            string rStr = "";

            int ll = lineLength;
            int nc = lStr.Length;
            int si = 0;
            int cl = ll;

            while (nc > 0)
            {
                if (si + cl > nc)
                    cl = nc - si;
                if (cl > 0)
                {
                    rStr += lStr.Substring(si, cl) + (char)13 + (char)10;
                    //rStr += (char)13 + (char)10;                    
                }
                else
                {
                    break;
                }

                si += cl;
                nc -= cl;
            }

            return rStr;
        }

        public static char GetReadableCharByValue(byte val)
        {
            char rc = '\0';
            if (val < 10)
            {
                rc = (char)(48 + val);
            }
            else if (val < 36)
            {
                rc = (char)(val + 55);
            }
            else if (val < 38)
            {
                rc = (char)(val - 1);
            }
            else
            {
                rc = (char)(val + 59);
            }

            return rc;
        }

        public static byte GetValueFromReadableChar(char rc)
        {
            int val = 0;

            int tv = (byte)rc;
            if (tv < 37)
            {
                val = tv + 1;
            }
            else if (tv < 58)
            {
                val = tv - 48;
            }
            else if (tv < 91)
            {
                val = tv - 55;
            }
            else
            {
                val = tv - 59;
            }

            return (byte)val;
        }

        public static string ConvertByteArrayToReadableString(byte[] data)
        {
            string rsData = "";

            byte lowMask = 63;
            byte highMask = 192;

            int cbitp = 0;

            for (int i = 0; i < data.Length; i++)
            {
                byte cb = data[i];

                byte lowBits = (byte)(cb & lowMask);

                cbitp += 6;

                byte highBits = (byte)((cb & highMask) >> 6);

            }

            return rsData;
        }

        public static string GetCurCultureNumberString(string nstr)
        {
            string cs = CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
            return nstr.Replace(".", cs).Replace(",", cs);
        }


        private const int buffer_size = 100;

        public static byte[] DataComp(byte[] data)
        {
            MemoryStream ms = new MemoryStream();
            DeflateStream sdStr = new DeflateStream(ms, CompressionMode.Compress, true);
            sdStr.Write(data, 0, data.Length);
            sdStr.Close();
            ms.Position = 0;
            byte[] rd = new byte[ms.Length];
            ms.Read(rd, 0, (int)ms.Length);
            ms.Close();
            sdStr.Dispose();
            ms.Dispose();
            return rd;
        }

        public static byte[] DataDecomp(byte[] data)
        {
            MemoryStream ms = new MemoryStream(data);
            ms.Position = 0;
            DeflateStream dStr = new DeflateStream(ms, CompressionMode.Decompress);
            byte[] buff = new byte[ms.Length + buffer_size * 10];
            int nbr = ReadAllBytesFromStream(dStr, buff);
            byte[] dd = new byte[nbr];
            Array.Copy(buff, dd, nbr);
            buff = null;
            return dd;
        }

        public static int ReadAllBytesFromStream(Stream stream, byte[] buffer)
        {
            // Use this method is used to read all bytes from a stream.
            int offset = 0;
            int totalCount = 0;
            while (true)
            {
                int bytesRead = stream.Read(buffer, offset, buffer_size);
                if (bytesRead == 0)
                {
                    break;
                }
                offset += bytesRead;
                totalCount += bytesRead;
            }
            return totalCount;
        }

        public static string StringComp(string data)
        {
            return Conv.StringFromByteArray(Misc.DataComp(Conv.ByteArrayFromString(data)));
        }

        public static string StringDecomp(string data)
        {
            return Conv.StringFromByteArray(Misc.DataDecomp(Conv.ByteArrayFromString(data)));
        }
			
		public float ClampValue(float val) {
			if (val < 0)
				return 0;
			if (val > 1)
				return 1;
			return val;
		}

		public static int LimitValue(int val, int min, int max)
		{
			if (val < min)
				return min;
			if (val > max)
				return max;
			return val;
		}

        public static float LimitValue(float val, float min, float max)
        {
            if (val < min)
                return min;
            if (val > max)
                return max;
            return val;
        }

        public static float MapValue(float val, float inMin, float inMax, float outMin, float outMax)
        {
            float mv = LimitValue(val, inMin, inMax);

            float ir = inMax - inMin;

            mv -= inMin;
            mv /= ir;

            float or = outMax - outMin;

            return mv * or + outMin;
        }

        public static string GetPath(string filePath)
        {
            string fp = filePath;
            int lpi = fp.LastIndexOf('\\');
            if (lpi > 0)
                fp = fp.Substring(0, lpi);
            return fp;
        }

        public static string GetFileName(string filePath)
        {
            string fn = filePath;
            int lpi = fn.LastIndexOf('\\');
            if (lpi > 0)
                fn = fn.Substring(lpi + 1);
            return fn;
        }

        public static string GetExtensionFromPath(string fp)
        {
            int wi = fp.LastIndexOf('.');
            if (wi >= 0)
                return fp.Substring(wi + 1).ToLower();
            return fp.ToLower();
        }

        public static string RemoveFileExtension(string fileName)
        {
            string fn = fileName;

            int li = fn.LastIndexOf('.');
            if (li > 0)
                fn = fn.Substring(0, li);

            return fn;
        }

        public static string GetTimeString(long dsTime)
        {
            return GetTimeString(dsTime, "", "00", true, 1.0f);
        }

        public static string GetTimeString(long dsTime, string msFormat)
        {
            return GetTimeString(dsTime, msFormat, "00", false, 1.0f);
        }

        public static string GetTimeString(long dsTime, string msFormat, string valFormat, bool useHours, float msFactor)
        {
            string ts = "";

            long totalSeconds = (long)(dsTime * 0.0000001f); // 10000000;
            long totalMiliseconds = (long)(dsTime * 0.0001f) - (totalSeconds * 1000);
            long totalMinutes = 0;
            long totalHours = 0;

            if (totalSeconds > 59)
            {
                totalMinutes = totalSeconds / 60;
                totalSeconds -= totalMinutes * 60;

                if (totalMinutes > 59)
                {
                    totalHours = totalMinutes / 60;
                    totalMinutes -= totalHours * 60;
                }
            }

            if (!useHours)
            {
                totalMinutes += totalHours * 60;
                ts = totalMinutes.ToString(valFormat) + ":" + totalSeconds.ToString(valFormat) + "." + ((int)(totalMiliseconds * msFactor)).ToString(msFormat);
            }
            else
            {
                ts = totalHours.ToString(valFormat) + ":" + totalMinutes.ToString(valFormat) + ":" + totalSeconds.ToString(valFormat) + "." + ((int)(totalMiliseconds * msFactor)).ToString(msFormat);
            }


            return ts;
        }

        public static object GetObjectFromData(byte[] valData, Type vType, object initObj)
        {
			//string[] vs;
            switch (vType.Name)
            {                
                case "Single":
                    return Conv.SingleFromData(valData);
               default:

					//Debug.WriteLine("Error creating type from data: " + vType.Name);
                    return null;
            }
        }

        public static object GetObjectFromString(string val, Type vType, object initObj)
        {
			//string[] vs;
            switch (vType.Name)
            {
                case "FormWindowState":
                    return Enum.Parse(vType, val);
                case "Boolean":
                    return val == "True";
                case "String":
                    return (string)val;
                case "Int32":
                    return Convert.ToInt32(val);
                case "Int64":
                    return Convert.ToInt64(val);                
                default:
                    return null;
            }
        }

        public static string GetStringFromValue(object val)
        {
            Type vType = val.GetType();
            string vs;
            switch (vType.Name)
            {
                default:
                    vs = val.ToString();
                    break;
            }
            return vs;
        }

        public static string GetFileNameFromPath(string filePath)
        {
            return GetFileNameFromPath(filePath, true);
        }

        public static string GetFileNameFromPath(string filePath, bool removeExtension)
        {
            string fn = filePath;
            int lpi = fn.LastIndexOf('\\');
            if (lpi >= 0)
                fn = fn.Substring(lpi + 1);

            if (removeExtension)
            {
                int exi = fn.LastIndexOf('.');
                if (exi >= 0)
                    fn = fn.Substring(0, exi);
            }

            return fn;
        }
    }

}
